@@ -71,13 +71,15 @@ class Invalid(Exception):
         self.children = []
 
     def messages(self):
-        """ Return an iterable of error messages for this exception
-        using the ``msg`` attribute of this error node.  If the
-        ``msg`` attribute is iterable, it is returned.  If it is not
-        iterable, a single-element list containing the ``msg`` value
-        is returned."""
+        """ Return an iterable of error messages for this exception using the
+        ``msg`` attribute of this error node.  If the ``msg`` attribute is
+        iterable, it is returned.  If it is not iterable, and is
+        non-``None``, a single-element list containing the ``msg`` value is
+        returned.  If the value is ``None``, an empty list is returned."""
         if is_nonstr_iter(self.msg):
             return self.msg
+        if self.msg is None:
+            return []
         return [self.msg]
 
     def add(self, exc, pos=None):
@@ -159,7 +161,7 @@ class Invalid(Exception):
             keyparts = []
             msgs = []
             for exc in path:
-                exc.msg and msgs.append(exc.msg)
+                exc.msg and msgs.extend(exc.messages())
                 keyname = exc._keyname()
                 keyname and keyparts.append(keyname)
             errors['.'.join(keyparts)] = '; '.join(interpolate(msgs))
@@ -177,15 +179,18 @@ class All(object):
         self.validators = validators
 
     def __call__(self, node, value):
-        msgs = []
+        excs = []
         for validator in self.validators:
             try:
                 validator(node, value)
             except Invalid as e:
-                msgs.append(e.msg)
+                excs.append(e)
 
-        if msgs:
-            raise Invalid(node, msgs)
+        if excs:
+            exc = Invalid(node, [exc.msg for exc in excs])
+            for e in excs:
+                exc.children.extend(e.children)
+            raise exc
 
 class Function(object):
     """ Validator which accepts a function and an optional message;
@@ -220,9 +225,9 @@ class Function(object):
     def __call__(self, node, value):
         result = self.function(value)
         if not result:
-            raise Invalid(node, self.message)
+            raise Invalid(node, translationstring.TranslationString(self.message, mapping={'val':value}))
         if isinstance(result, string_types):
-            raise Invalid(node, result)
+            raise Invalid(node, translationstring.TranslationString(result, mapping={'val':value}))
 
 class Regex(object):
     """ Regular expression validator.
@@ -493,7 +498,10 @@ class Mapping(SchemaType):
         if listitem:
             selfprefix = prefix
         else:
-            selfprefix = '%s%s.' % (prefix, node.name)
+            if node.name:
+                selfprefix = '%s%s.' % (prefix, node.name)
+            else:
+                selfprefix = prefix
 
         for subnode in node.children:
             name = subnode.name
@@ -908,7 +916,7 @@ class String(SchemaType):
             return result
         except Exception as e:
             raise Invalid(node,
-                          _('"${val} cannot be serialized: ${err}',
+                          _('${val} cannot be serialized: ${err}',
                             mapping={'val':appstruct, 'err':e})
                           )
     def deserialize(self, node, cstruct):
@@ -926,7 +934,7 @@ class String(SchemaType):
                 result = text_type(cstruct)
         except Exception as e:
             raise Invalid(node,
-                          _('${val} is not a string: %{err}',
+                          _('${val} is not a string: ${err}',
                             mapping={'val':cstruct, 'err':e}))
 
         return result
@@ -1686,7 +1694,8 @@ class _SchemaMeta(type):
         nodes = []
         for name, value in clsattrs.items():
             if isinstance(value, SchemaNode):
-                value.name = name
+                if not value.name:
+                    value.name = name
                 if value.raw_title is _marker:
                     value.title = name.replace('_', ' ').title()
                 nodes.append((value._order, value))
@@ -1756,7 +1765,10 @@ def _unflatten_mapping(node, paths, fstruct,
         def rewrite_subpath(subpath):
             return subpath
     node_name = node.name
-    prefix = node_name + '.'
+    if node_name:
+        prefix = node_name + '.'
+    else:
+        prefix = ''
     prefix_len = len(prefix)
     appstruct = {}
     subfstruct = {}
