@@ -91,6 +91,51 @@ class TestInvalid(unittest.TestCase):
         self.assertEqual(d, {'node1.node2.3': 'exc1; exc2; exc3',
                              'node1.node4': 'exc1; exc4'})
 
+    def test_asdict_with_all_validator(self):
+        # see https://github.com/Pylons/colander/pull/27
+        from colander import All
+        from colander import Positional
+        node1 = DummySchemaNode(None, 'node1')
+        node2 = DummySchemaNode(Positional(), 'node2')
+        node3 = DummySchemaNode(Positional(), 'node3')
+        node1.children = [node3]
+        validator1 = DummyValidator('validator1')
+        validator2 = DummyValidator('validator2')
+        validator = All(validator1, validator2)
+        exc1 =  self._makeOne(node1, 'exc1')
+        exc1.pos = 1
+        exc1['node3'] = 'message1'
+        exc2 = self._makeOne(node2, 'exc2')
+        exc3 = invalid_exc(validator, None, None)
+        exc1.add(exc2, 2)
+        exc2.add(exc3, 3)
+        d = exc1.asdict()
+        self.assertEqual(d, 
+                         {'node1.node2.3': 'exc1; exc2; validator1; validator2',
+                          'node1.node3': 'exc1; message1'})
+
+    def test_asdict_with_all_validator_functional(self):
+        # see https://github.com/Pylons/colander/issues/2
+        import colander as c
+        class MySchema(c.MappingSchema):
+            number1 = c.SchemaNode(c.Int(), validator=c.Range(min=1))
+            number2 = c.SchemaNode(c.Int(), validator=c.Range(min=1))
+        def validate_higher(node, val):
+            if val['number1'] >= val['number2']:
+                raise c.Invalid(node, 'Number 1 must be lower than number 2')
+        def validate_different(node, val):
+            if val['number1'] == val['number2']:
+                raise c.Invalid(node, "They can't be the same, either")
+        schema = MySchema(validator=c.All(validate_higher, validate_different))
+        try:
+            schema.deserialize(dict(number1=2, number2=2))
+        except c.Invalid as e:
+            result = e.asdict()
+            self.assertEqual(
+                result,
+                {'': ("Number 1 must be lower than number 2; "
+                      "They can't be the same, either")})
+
     def test___str__(self):
         from colander import Positional
         node1 = DummySchemaNode(None, 'node1')
@@ -138,6 +183,11 @@ class TestInvalid(unittest.TestCase):
         exc = self._makeOne(node, 'msg')
         self.assertEqual(exc.messages(), ['msg'])
 
+    def test_messages_msg_None(self):
+        node = DummySchemaNode(None)
+        exc = self._makeOne(node, None)
+        self.assertEqual(exc.messages(), [])
+        
 class TestAll(unittest.TestCase):
     def _makeOne(self, validators):
         from colander import All
@@ -156,6 +206,19 @@ class TestAll(unittest.TestCase):
         e = invalid_exc(validator, None, None)
         self.assertEqual(e.msg, ['msg1', 'msg2'])
 
+    def test_Invalid_children(self):
+        from colander import Invalid
+        node1 = DummySchemaNode(None, 'node1')
+        node = DummySchemaNode(None, 'node')
+        node.children = [node1]
+        exc1 = Invalid(node1, 'exc1')
+        exc2 = Invalid(node1, 'exc2')
+        validator1 = DummyValidator('validator1', [exc1])
+        validator2 = DummyValidator('validator2', [exc2])
+        validator = self._makeOne([validator1, validator2])
+        exc = invalid_exc(validator, node, None)
+        self.assertEqual(exc.children, [exc1, exc2])
+        
 class TestFunction(unittest.TestCase):
     def _makeOne(self, *arg, **kw):
         from colander import Function
@@ -180,6 +243,30 @@ class TestFunction(unittest.TestCase):
         e = invalid_exc(validator, None, None)
         self.assertEqual(e.msg, 'fail')
 
+    def test_error_message_adds_mapping_to_configured_message(self):
+        validator = self._makeOne(lambda x: False, message='fail ${val}')
+        e = invalid_exc(validator, None, None)
+        self.assertEqual(e.msg.interpolate(), 'fail None')
+
+    def test_error_message_adds_mapping_to_return_message(self):
+        validator = self._makeOne(lambda x: 'fail ${val}')
+        e = invalid_exc(validator, None, None)
+        self.assertEqual(e.msg.interpolate(), 'fail None')
+
+    def test_error_message_does_not_overwrite_configured_domain(self):
+        import translationstring
+        _ = translationstring.TranslationStringFactory('fnord')
+        validator = self._makeOne(lambda x: False, message=_('fail ${val}'))
+        e = invalid_exc(validator, None, None)
+        self.assertEqual(e.msg.domain, 'fnord')
+
+    def test_error_message_does_not_overwrite_returned_domain(self):
+        import translationstring
+        _ = translationstring.TranslationStringFactory('fnord')
+        validator = self._makeOne(lambda x: _('fail ${val}'))
+        e = invalid_exc(validator, None, None)
+        self.assertEqual(e.msg.domain, 'fnord')
+
     def test_propagation(self):
         validator = self._makeOne(lambda x: 'a' in x, 'msg')
         self.assertRaises(TypeError, validator, None, None)
@@ -2039,6 +2126,18 @@ class TestSchemaNode(unittest.TestCase):
         self.assertEqual(len(outer_clone.children), 0)
         self.assertEqual(len(outer_node.children), 1)
 
+    def test_declarative_name_reassignment(self):
+        # see https://github.com/Pylons/colander/issues/39
+        import colander
+        class FnordSchema(colander.Schema):
+            fnord = colander.SchemaNode(
+                colander.Sequence(), 
+                colander.SchemaNode(colander.Integer(), name=''), 
+                name="fnord[]"
+                )
+        schema = FnordSchema()
+        self.assertEqual(schema['fnord[]'].name, 'fnord[]')
+        
 class TestDeferred(unittest.TestCase):
     def _makeOne(self, wrapped):
         from colander import deferred
@@ -2193,6 +2292,42 @@ class TestFunctional(object):
         for k, v in result.items():
             self.assertEqual(expected[k], v)
 
+    def test_flatten_mapping_has_no_name(self):
+        import colander
+        appstruct = {
+            'int':10,
+            'ob':colander.tests,
+            'seq':[(1, 's'),(2, 's'), (3, 's'), (4, 's')],
+            'seq2':[{'key':1, 'key2':2}, {'key':3, 'key2':4}],
+            'tup':(1, 's'),
+            }
+        schema = self._makeSchema(name='')
+        result = schema.flatten(appstruct)
+
+        expected = {
+            'seq.2.tupstring': 's',
+            'seq2.0.key2': 2,
+            'ob': colander.tests,
+            'seq2.1.key2': 4,
+            'seq.1.tupstring': 's',
+            'seq2.0.key': 1,
+            'seq.1.tupint': 2,
+            'seq.0.tupstring': 's',
+            'seq.3.tupstring': 's',
+            'seq.3.tupint': 4,
+            'seq2.1.key': 3,
+            'int': 10,
+            'seq.0.tupint': 1,
+            'tup.tupint': 1,
+            'tup.tupstring': 's',
+            'seq.2.tupint': 3,
+        }
+
+        for k, v in expected.items():
+            self.assertEqual(result[k], v)
+        for k, v in result.items():
+            self.assertEqual(expected[k], v)
+            
     def test_unflatten_ok(self):
         import colander
         fstruct = {
@@ -2229,6 +2364,42 @@ class TestFunctional(object):
         for k, v in result.items():
             self.assertEqual(expected[k], v)
 
+    def test_unflatten_mapping_no_name(self):
+        import colander
+        fstruct = {
+            'seq.2.tupstring': 's',
+            'seq2.0.key2': 2,
+            'ob': colander.tests,
+            'seq2.1.key2': 4,
+            'seq.1.tupstring': 's',
+            'seq2.0.key': 1,
+            'seq.1.tupint': 2,
+            'seq.0.tupstring': 's',
+            'seq.3.tupstring': 's',
+            'seq.3.tupint': 4,
+            'seq2.1.key': 3,
+            'int': 10,
+            'seq.0.tupint': 1,
+            'tup.tupint': 1,
+            'tup.tupstring': 's',
+            'seq.2.tupint': 3,
+        }
+        schema = self._makeSchema(name='')
+        result = schema.unflatten(fstruct)
+
+        expected = {
+            'int':10,
+            'ob':colander.tests,
+            'seq':[(1, 's'),(2, 's'), (3, 's'), (4, 's')],
+            'seq2':[{'key':1, 'key2':2}, {'key':3, 'key2':4}],
+            'tup':(1, 's'),
+            }
+
+        for k, v in expected.items():
+            self.assertEqual(result[k], v)
+        for k, v in result.items():
+            self.assertEqual(expected[k], v)
+            
     def test_flatten_unflatten_roundtrip(self):
         import colander
         appstruct = {
@@ -2433,13 +2604,16 @@ class DummySchemaNode(object):
                 return child
 
 class DummyValidator(object):
-    def __init__(self, msg=None):
+    def __init__(self, msg=None, children=None):
         self.msg = msg
+        self.children = children
 
     def __call__(self, node, value):
         from colander import Invalid
         if self.msg:
-            raise Invalid(node, self.msg)
+            e = Invalid(node, self.msg)
+            self.children and e.children.extend(self.children)
+            raise e
 
 class Uncooperative(object):
     def __str__(self):
